package com.example.demo.controller;

import org.springframework.web.bind.annotation.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import java.util.List;
import jakarta.servlet.http.HttpSession;
import com.example.demo.entity.Board;
import com.example.demo.entity.User;
import com.example.demo.service.BoardService;
import com.example.demo.service.UserService;
import java.util.Arrays;



@RestController //json 형태로 반환
@RequestMapping("/boards")// board mapping (get,post 방식으로 url 요청됨) 
public class BoardController {

    private final BoardService boardService;
    //유저 삭제시 유저가 작성한 글도 삭제하기 위해 유저 서비스도 참조 
    private final UserService userService;

    @Autowired //타입에 해당하는 빈을 찾아줌 -생성자, setter, 필드 Userservice에서 받아옴
    public BoardController(BoardService boardService, UserService userService) {
        this.boardService = boardService;
        this.userService = userService;
    }

    // 전체 게시글 가져오기
    @GetMapping
    public List<Board> getBoards() {
        return boardService.getAllBoards();
    }

    // 특정 사용자의 게시글 개수 가져오기
    @GetMapping("/count/{userId}")
    public int getPostCountByUserId(@PathVariable String userId) {
        return boardService.getPostCountByUserId(userId);
    }

    // 특정 사용자의 게시글 목록 가져오기
    @GetMapping("/user/{userId}")
    public List<Board> getPostsByUserId(@PathVariable String userId) {
        return boardService.getPostsByUserId(userId);
    }

    // 게시글 생성 (로그인된 사용자 정보 사용)
    @PostMapping("/create")
    public ResponseEntity<?> createPost(@RequestBody Board board, HttpSession session) {
        try {
            // 현재 로그인된 사용자 가져오기     // HTML특정 요소 속성 가져오기
            User loggedInUser = (User) session.getAttribute("user");
            if (loggedInUser == null) {
                //미승인 시 리턴
                return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("로그인이 필요합니다.");
            }

            // 게시글에 현재 로그인된 사용자 설정
            board.setUser(loggedInUser);

            // 게시글 저장
            Board createdBoard = boardService.saveBoard(board, loggedInUser.getUserId());
            return ResponseEntity.ok("게시글 작성 완료");
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR)
                                .body("게시글 작성 중 오류 발생: " + e.getMessage());
        }
    }

    // Board 게시글 생성 시 로그인된 사용자의 ID를 사용하여 CRT_USER, UDT_USER 설정
    //로그인 된 유저확인, 게시글 작성시 로그인 유저 정보를 불러와 저장
    @PostMapping
    public ResponseEntity<Board> createBoard(@RequestBody Board board, HttpSession session) {
        String loggedInUser = (String) session.getAttribute("userId");
        board.setUpdatedBy(loggedInUser);
        Board createdBoard = boardService.saveBoard(board, loggedInUser);
        return new ResponseEntity<>(createdBoard, HttpStatus.CREATED);
    }

    // Board 수정 시 UDT_USER 업데이트
    // 게시글 내용 수정시 db에 수정자 변경
    @PutMapping("/{id}")
    public ResponseEntity<Board> updateBoard(@PathVariable Long id, @RequestBody Board boardDetails, HttpSession session) {
        String loggedInUser = (String) session.getAttribute("userId");
        boardDetails.setUpdatedBy(loggedInUser);
        Board updatedBoard = boardService.saveBoard(boardDetails, loggedInUser);
        return ResponseEntity.ok(updatedBoard);
    }

    // 카테고리별 게시글 리스트 반환
    @GetMapping("/category/{category}")
    public List<Board> getBoardsByCategory(
        @PathVariable("category") String category) {
            //all일때 게시판 테이블 내 데이터 다 가져와서 보여줌
        if (category.equals("all")) {
            return boardService.getAllBoards();
        } else { // 아닐때 카테고리 아이디 받아와서 카테고리 넘버에 맞는 데이터 보여줌
            int categoryId = getCategoryId(category);
            return boardService.getBoardsByCategory(categoryId);
        }
    }
    
    // 게시판 카테고리 분류
    private int getCategoryId(String category) { //string 카테고리 값을 받아와서 int형으로 바꾸기
        switch (category) {
            case "free":
                return 2;
            case "questions":
                return 3;
            case "notice":
                return 4;
            default:
                return 1;
        }
    }

    //글 세부조회
    @GetMapping("/detail/{boardNumber}")
    public ResponseEntity<Board> getBoardDetail(@PathVariable Long boardNumber) {
        Board board = boardService.getBoardDetail(boardNumber);
        return new ResponseEntity<>(board, HttpStatus.OK);
    }
    //글 수정
    @PutMapping("/update/{boardNumber}")
    public ResponseEntity<Board> updateBoard(
            @PathVariable Long boardNumber,
            @RequestBody Board updatedBoard) {
        Board board = boardService.updateBoard(boardNumber, updatedBoard);
        return new ResponseEntity<>(board, HttpStatus.OK);
    }
    //글 삭제
    @DeleteMapping("/delete/{boardNumber}")
    public ResponseEntity<Void> deleteBoard(@PathVariable Long boardNumber) {
        boardService.deleteBoard(boardNumber);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

}